﻿Imports C1.WPF.C1Chart

Class MainWindow
    Dim rnd As New Random()

    Private Sub Window_Loaded(sender As System.Object, e As System.Windows.RoutedEventArgs)
        CreateSampleChart(5, 100)
    End Sub

    Private Sub chart_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles chart.Loaded
        UpdateList()
    End Sub

    Private Sub CreateSampleChart(nseries As Integer, npoints As Integer)
        chart.BeginUpdate()
        chart.Reset(True)
        For i As Integer = 0 To nseries - 1
            Dim xvalues = New Double(npoints - 1) {}
            Dim yvalues = New Double(npoints - 1) {}
            For j As Integer = 0 To npoints - 1
                xvalues(j) = rnd.[Next](0, 100)
                yvalues(j) = rnd.[Next](0, 100)
            Next

            Dim ds As XYDataSeries = New XYDataSeries()
            ds.XValuesSource = xvalues
            ds.ValuesSource = yvalues
            chart.Data.Children.Add(ds)
        Next

        chart.ChartType = ChartType.XYPlot

        chart.Actions.Add(New ZoomAction())

        Dim _scaleAction As New ScaleAction()
        _scaleAction.Modifiers = ModifierKeys.Control
        chart.Actions.Add(_scaleAction)

        Dim _translateAction As New TranslateAction()
        _translateAction.Modifiers = ModifierKeys.Shift
        chart.Actions.Add(_translateAction)
        chart.EndUpdate()
    End Sub

    Private Sub chart_ActionLeave(sender As Object, e As System.EventArgs) Handles chart.ActionLeave
        UpdateList()
    End Sub

    Private Sub Button_Click(sender As System.Object, e As System.Windows.RoutedEventArgs)
        chart.View.AxisX.Scale = 1
        chart.View.AxisY.Scale = 1
        UpdateList()
    End Sub

    Private Sub UpdateList()
        Dim list = GetPointsInView(chart)
        listBox.ItemsSource = list
        textBlock.Text = String.Format("{0} points", list.Count)
    End Sub

    Private Function GetPointsInView(chart As C1Chart) As List(Of Point)
        Dim list = New List(Of Point)()
        Dim nser As Integer = chart.Data.Children.Count
        Dim xmin = GetAxisMin(chart.View.AxisX)
        Dim xmax = GetAxisMax(chart.View.AxisX)
        Dim ymin = GetAxisMin(chart.View.AxisY)
        Dim ymax = GetAxisMax(chart.View.AxisY)

        For i As Integer = 0 To nser - 1
            Dim ds = TryCast(chart.Data.Children(i), XYDataSeries)
            If ds IsNot Nothing Then
                ' fast access to data values
                Dim values = DirectCast(ds, IDataSeriesInfo).GetValues()
                If values IsNot Nothing Then
                    Dim len = values.GetLength(1)
                    For ipts As Integer = 0 To len - 1
                        Dim x = values(1, ipts)
                        Dim y = values(0, ipts)
                        If Not Double.IsNaN(x) AndAlso Not Double.IsNaN(y) Then
                            If xmin <= x AndAlso x <= xmax AndAlso ymin <= y AndAlso y <= ymax Then
                                list.Add(New Point(x, y))
                            End If
                        End If
                    Next
                End If
            End If
        Next
        Return list
    End Function

    Private Function GetAxisMin(ax As Axis) As Double
        Dim range As Double = (ax.ActualMax - ax.ActualMin) * ax.Scale
        Return ax.ActualMin + ax.Value * (ax.ActualMax - ax.ActualMin - range)
    End Function

    Private Function GetAxisMax(ax As Axis) As Double
        Dim range As Double = (ax.ActualMax - ax.ActualMin) * ax.Scale
        Return ax.ActualMin + ax.Value * (ax.ActualMax - ax.ActualMin - range) + range
    End Function
End Class
